/*
 * (c) Copyright 2010-2011 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity;

import java.io.Closeable;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.databene.commons.FileUtil;
import org.databene.commons.IOUtil;
import org.databene.commons.LoggerEscalator;
import org.databene.commons.file.FilePrintStream;
import org.databene.commons.iterator.TableRowIterator;
import org.databene.dbsanity.model.SanityCheck;
import org.databene.dbsanity.model.SanityCheckFolder;
import org.databene.dbsanity.model.SanityCheckSuite;
import org.databene.dbsanity.report.ReportContext;
import org.databene.dbsanity.report.ReportModule;
import org.databene.dbsanity.report.ReportScope;
import org.databene.dbsanity.report.ReportUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Encapsulates all classes and functionality concerned with report creation.<br/><br/>
 * Created: 13.10.2010 14:41:10
 * @since 1.0
 * @author Volker Bergmann
 */
public class Reporter implements Closeable {

	private static final Logger LOGGER = LoggerFactory.getLogger(DbSanity.class);
	
	private String style;
	private List<ReportModule> modules;
	private LoggerEscalator escalator = new LoggerEscalator();

	private ReportContext context;

	public Reporter(File reportFolder, String style, ReportContext context) throws IOException {
		this.style = style;
		this.context = context;
		this.modules = new ArrayList<ReportModule>();
	}
	
	// interface -------------------------------------------------------------------------------------------------------
	
	public void addModule(ReportModule module) {
		if (context != null)
			module.setContext(context);
		module.setStyle(style);
		this.modules.add(module);
	}
	
	public List<ReportModule> getModules() {
		return modules;
	}
	
	public void report(SanityCheck check, TableRowIterator failureIterator) {
	    if (!failureIterator.hasNext())
	    	success(check);
	    else {
	    	int count = 0;
	    	int defectCountLimit = context.getDefectCountLimit();
			while (failureIterator.hasNext() && (defectCountLimit  <= 0 || count < defectCountLimit )) {
	    		failure(check, failureIterator.next(), failureIterator.getColumnLabels());
	    		count++;
	    	}
	    }
    }
	
	// ReportModule interface implementation ---------------------------------------------------------------------------

	public void setContext(ReportContext context) {
		this.context = context;
	    for (ReportModule module : modules)
	    	module.setContext(context);
    }

	public void success(SanityCheck check) {
	    LOGGER.debug("Check successful: '{}'", check.getName());
	    for (ReportModule module : modules)
	    	module.success(check);
    }

	public void failure(SanityCheck check, Object[] errorInfo, String[] infoHeaders) {
	    escalator.escalate("Check failed", check, errorInfo);
	    for (ReportModule module : modules)
	    	module.failure(check, errorInfo, infoHeaders);
    }

	public void error(SanityCheck check, Exception e) {
	    LOGGER.error("Error in check: '{}'", check.getName());
	    for (ReportModule module : modules)
	    	module.error(check, e);
    }

	public void skipped(SanityCheck check) {
	    LOGGER.error("Skipped check: '{}'", check.getName());
	    for (ReportModule module : modules)
	    	module.skipped(check);
	}

	public void summary(SanityCheckSuite rootSuite) {
	    for (ReportModule module : modules)
	    	module.summary(rootSuite);
	    createDashboardPage(rootSuite, true);
	}
	
	public void close() {
	    for (ReportModule module : modules)
	    	IOUtil.close(module);
    }
	
	// private helpers -------------------------------------------------------------------------------------------------

	private void createDashboardPage(SanityCheckSuite suite, boolean isRoot) {
		try {
			FileUtil.ensureDirectoryExists(suite.getReportFolder());
			String title = "DB Sanity Report '" + suite.getName() + "'";
			if (context.getAppVersion() != null)
				title += " (v." + context.getAppVersion() + ")";
			File file = new File(suite.getReportFolder(), "index.html");
			FilePrintStream out = ReportUtil.openNewFile(file, title, new File(suite.getReportFolder(), style + ".css"));
			
			ReportScope scope = (isRoot ? ReportScope.ROOT : ReportScope.FOLDER);
			out.println("	<table width='90%' cellpadding='0' cellspacing='8'>");
			int moduleIndex = 0;
			while (moduleIndex < modules.size()) {
				int nextWideModuleIndex = nextWideModuleIndex(scope, moduleIndex);
				
				// render next small modules
				List<ReportModule> nextModules = sortSmallModulesForDisplay(scope, moduleIndex, nextWideModuleIndex - 1);
				if (nextModules.size() > 0) {
					out.println("		<tr>");
					out.println("			<td class='mainCell' valign='top' width='60%'>");
					for (int i = 0; i < nextModules.size(); i += 2)
						nextModules.get(i).renderDashboardView(scope, suite, out);
					out.println("			</td>");
					out.println("			<td class='mainCell' valign='top' width='40%'>");
					for (int i = 1; i < nextModules.size(); i += 2)
						nextModules.get(i).renderDashboardView(scope, suite, out);
					out.println("			</td>");
					out.println("		<tr>");
				}
	
				if (nextWideModuleIndex < modules.size()) {
					ReportModule module = modules.get(nextWideModuleIndex);
					out.println("		<tr class='mainCell'>");
					out.println("			<td colspan='2'>");
					module.renderDashboardView(scope, suite, out);
					out.println("			</td>");
					out.println("		</tr>");
				}
				
				moduleIndex = nextWideModuleIndex + 1;
			}
			out.println("	</table>");
			out.println();
	
			out.println(context.footer());
			out.println("	</center>");
		    out.println("</body>");
		    out.println("</html>");
		    IOUtil.close(out);
	        
		    // if the suite is a folder, create dashboard pages for the child suites
		    if (suite instanceof SanityCheckFolder) {
		    	SanityCheckFolder folder = (SanityCheckFolder) suite;
		        if (folder.getChildSuites().size() > 0)
		        	for (SanityCheckSuite child : folder.getChildSuites().values())
		        		if (child instanceof SanityCheckFolder)
		        			createDashboardPage(child, false);
		    }
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}

	private int nextWideModuleIndex(ReportScope scope, int moduleIndex) {
		for (int i = moduleIndex; i < modules.size(); i++)
			if (modules.get(i).getDashboardColSpan(scope) == 2)
				return i;
		return modules.size();
	}

	private List<ReportModule> sortSmallModulesForDisplay(ReportScope scope, int first, int last) {
		List<ReportModule> result = new ArrayList<ReportModule>();
		for (int i = first; i <= last; i++) {
			ReportModule module = modules.get(i);
			if (module.getDashboardColSpan(scope) == 1)
				result.add(module);
		}
		return result;
	}

}
