/*
 * (c) Copyright 2010 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.model;

import java.util.Iterator;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Sends a tree of the test suite hierarchy to the logger.<br/><br/>
 * Created: 28.10.2010 10:42:18
 * @since 0.6.4
 * @author Volker Bergmann
 */
public class TestSuiteLogger {
	
	private static final Logger LOGGER = LoggerFactory.getLogger(SuiteVisitor.class);

	String indent = "";
	
	// interface ---------------------------------------------------------------
	
	public void log(SanityCheckSuite suite) {
	    log(suite, false);
    }
	
	// private helper methods --------------------------------------------------

	private void log(SanityCheckSuite suite, boolean hasSiblings) {
	    if (suite instanceof SanityCheckFolder)
	    	log((SanityCheckFolder) suite, hasSiblings);
	    else
	    	log((SanityCheckFile) suite, hasSiblings);
    }

	private void log(SanityCheckFolder folder, boolean hasSiblings) {
	    LOGGER.debug(indent + "D:" + folder.getName() + " " +
	    		stats(folder) +
	    		" (R:" + folder.getReportFolder() + ")"
	    	);
		increaseIndent(hasSiblings);
		Iterator<SanityCheckSuite> iterator = folder.getChildSuites().values().iterator();
		while (iterator.hasNext()) {
	    	SanityCheckSuite child = iterator.next();
	    	log(child, iterator.hasNext());
	    }
	    reduceIndent();
    }

	private void log(SanityCheckFile file, boolean hasSiblings) {
	    LOGGER.debug(indent + "F:" + file.toString() + ' ' + stats(file));
		increaseIndent(hasSiblings);
		for (SanityCheck check : file.getChecks().values())
			LOGGER.debug(indent + "C:" + check.getName() + ' ' +
		    		stats(check) + 
					" (R:" + check.getReportFolder() + ", " + 
					" T:" + check.getTmpResultFile() + ")"
				);
	    reduceIndent();
    }

	private String stats(SanityCheckSuite suite) {
	    return 	"[S:" + suite.getSuccessCount() + ", " +
	    		 "F:" + suite.getFailureCount() + ", " +
	    		 "E:" + suite.getErrorCount()   + ", " +
	    		 "D:" + suite.getDefectCount() + ", " +
		    	 "t:" + suite.getExecutionTime()  + "ms" + "]";
    }

	private String stats(SanityCheck check) {
	    return "[V:" + check.getVerdict()     + ", " +
	    		"D:" + check.getDefectCount() + ", " +
	    		"t:" + check.getExecutionTime() + "ms" + "]";
    }

	private void increaseIndent(boolean hasSiblings) {
	    if (indent.length() == 0)
	    	indent = "+-";
	    else if (hasSiblings)
	    	indent = indent.substring(0, indent.length() - 2) + "| " + indent.substring(indent.length() - 2);
	    else
	    	indent = indent.substring(0, indent.length() - 2) + "  " + indent.substring(indent.length() - 2);
    }

	private void reduceIndent() {
	    if (indent.length() >= 4)
	    	indent = indent.substring(0, indent.length() - 4) + indent.substring(indent.length() - 2);
	    else
	    	indent = "";
    }

}
