/*
 * (c) Copyright 2010 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.model.nk;

import java.io.File;
import java.io.IOException;
import java.sql.Connection;
import java.util.HashSet;

import org.databene.commons.HeavyweightIterator;
import org.databene.commons.iterator.TableRowIterator;
import org.databene.commons.math.Intervals;
import org.databene.commons.version.VersionNumber;
import org.databene.dbsanity.model.SanityCheck;
import org.databene.dbsanity.model.SanityCheckVerdict;
import org.databene.jdbacl.identity.IdentityModel;
import org.databene.jdbacl.identity.InvalidIdentityDefinitionError;
import org.databene.jdbacl.identity.KeyMapper;
import org.databene.jdbacl.identity.mem.GlobalRowId;
import org.databene.jdbacl.identity.mem.TargetTableMapper;

/**
 * {@link SanityCheck} implementation that verifies an {@link IdentityModel}
 * to be complete, unique and invertible.<br/><br/>
 * Created: 02.12.2010 16:46:27
 * @since 0.4
 * @author Volker Bergmann
 */
public class IdentityCheck extends SanityCheck {
	
	protected IdentityModel identity;
	protected String dbId;
	protected KeyMapper keyMapper;

	protected int tupleCount;

	public IdentityCheck(String name, IdentityModel identity, Intervals<VersionNumber> affectedVersions, 
			String dbId, File sourceFile, File tempFolder,
			File reportFolder, File docPage, KeyMapper keyMapper) {
		super(name, identity.getTable().getName(), affectedVersions, "identity", sourceFile, tempFolder, reportFolder, docPage);
		this.identity = identity;
		this.dbId = dbId;
		this.keyMapper = keyMapper;
		this.tupleCount = 0;
	}

	@Override
	public String getDetails() {
		return "Identity Check: " + identity.getDescription();
	}

	@Override
	public TableRowIterator query(Connection connection) {
		long start = System.nanoTime();
		try {
			TableRowIterator iterator = new DefectIterator(connection);
			this.verdict = (iterator.hasNext() ? SanityCheckVerdict.FAILED : SanityCheckVerdict.PASSED);
			return new Proxy(iterator);
		} catch (RuntimeException e) {
			this.verdict = SanityCheckVerdict.ERROR;
			throw e;
		} finally {
	    	int elapsed = (int) ((System.nanoTime() - start) / 1000000);
	    	this.executionTime = elapsed;
		}
	}


	
	class DefectIterator implements TableRowIterator {
		
		Connection connection;
		HeavyweightIterator<Object[]> nkPkIterator;
		TargetTableMapper tMapper;
		HashSet<GlobalRowId> pks;
		HashSet<GlobalRowId> reportedPksWithDuplicateNk;
		Object[] next;
		Object[] next2;
		
		public DefectIterator(Connection connection) {
			this.connection = connection;
			this.pks = new HashSet<GlobalRowId>();
			this.reportedPksWithDuplicateNk = new HashSet<GlobalRowId>();
			this.tMapper = new TargetTableMapper(keyMapper, connection, "__targetDB", identity);
			this.nkPkIterator = identity.createNkPkIterator(connection, dbId, keyMapper);
		}

		public boolean hasNext() {
			if (next == null)
				fetchNext();
			return (next != null);
		}
		
		public Object[] next() {
			Object[] result = next;
			next = next2;
			next2 = null;
			return result;
		}
		
		void fetchNext() {
			while (nkPkIterator.hasNext()) {
				Object[] candidate = nkPkIterator.next();
				if (candidate.length != identity.getTable().getPKColumnNames().length + 1)
					throw new InvalidIdentityDefinitionError("Invalid column count in NK-PK-Query");
				tupleCount++;
				String nk = identity.extractNK(candidate);
		    	Object pk = identity.extractPK(candidate);
		    	GlobalRowId globalRowId = new GlobalRowId(dbId, identity.getTable().getName(), pk);
				Object previousPK = tMapper.store(pk, nk);
		    	if (previousPK != null) { // check if the NK has already been used
		    		next = new Object[] { nk, pk, "Duplicate natural key" };
		    		reportedPksWithDuplicateNk.add(globalRowId);
			    	GlobalRowId prevGlobalRowId = new GlobalRowId(dbId, identity.getTable().getName(), previousPK);
		    		if (!reportedPksWithDuplicateNk.contains(prevGlobalRowId)) {
			    		next2 =  new Object[] { nk, previousPK, "Duplicate natural key" };
			    		reportedPksWithDuplicateNk.add(prevGlobalRowId);
		    		}
		    		return;
		    	} else
		    		
				if (!pks.add(globalRowId)) {
		    		next = new Object[] { "", pk, "Duplicate primary key" }; // TODO v0.x report PKs too
		    		return;
				}
		    	keyMapper.store(dbId, identity, nk, pk, null);
			}
			next = null;
			checkTupleCount();
		}

		private void checkTupleCount() {
			long rowCount = identity.getTable().getRowCount(connection);
			if (tupleCount != rowCount)
				throw new InvalidIdentityDefinitionError("Result set incomplete: Expected " + rowCount + " tuples but found " + tupleCount);
		}

		public String[] getColumnLabels() {
			return new String[] { "NK", "PK", "Error Message" }; // TODO v0.9 support composite keys
		}
		
		public void remove() {
			throw new UnsupportedOperationException(getClass().getSimpleName() + " does not support remove()");
		}

		public void close() throws IOException {
			nkPkIterator.close();
			keyMapper = null;
			tMapper = null;
			pks = null;
		}

	}

}
