/*
 * (c) Copyright 2011 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.parser;

import java.util.Set;

import org.databene.commons.CollectionUtil;
import org.databene.commons.StringUtil;
import org.databene.commons.SyntaxError;
import org.databene.commons.xml.XMLUtil;
import org.databene.dbsanity.model.query.DefectQueryCheck;
import org.databene.jdbacl.DatabaseDialect;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.w3c.dom.Element;

/**
 * Provides pattern-based checks for strings.<br/><br/>
 * Created: 18.10.2011 18:04:43
 * @since 0.8.4
 * @author Volker Bergmann
 */
public class PatternParser extends DbSanityXMLParser {
	
	private static final Logger LOGGER = LoggerFactory.getLogger(PatternParser.class);

	private static final Set<String> REQUIRED_ATTRIBUTES = CollectionUtil.toSet("expression");
	private static final Set<String> OPTIONAL_ATTRIBUTES = CollectionUtil.toSet("regex", "like");

	private static final String DEFAULT_DEFECT_TYPE = "regex";

	public PatternParser() {
		super("pattern", REQUIRED_ATTRIBUTES, OPTIONAL_ATTRIBUTES, DefectQueryCheck.class);
	}

	@Override
	public Object parse(Element element, Object[] parentPath, DbSanityParseContext context) {
		// check parent
	    DefectQueryCheck parent = (DefectQueryCheck) this.parent(parentPath);
		String table = parent.getTable();
		if (StringUtil.isEmpty(table))
			throw new SyntaxError("Missing 'table' spec in outer <check>", XMLUtil.format(element));

		// parse XML element
		String expression = getRequiredAttribute("expression", element);
		String regex = getRequiredAttribute("regex", element);
		String like  = getRequiredAttribute("like",  element);
		if (StringUtil.isEmpty(regex) && StringUtil.isEmpty(like))
			throw new SyntaxError("'regex' or 'like' must be specified in <pattern>", XMLUtil.format(element));
		// TODO v0.9 let the user enforce check execution on the client
		
		// create query
		DatabaseDialect dialect = context.getDialect();
		String defectQuery = "select * from " + table + " where ";
		if (!StringUtil.isEmpty(like))
			defectQuery += expression + " like " + like;
		if (!StringUtil.isEmpty(regex)) {
			if (dialect.supportsRegex()) {
				if (!StringUtil.isEmpty(like))
					defectQuery += " and ";
				defectQuery += dialect.regexQuery(expression, true, regex);
			} else { 
				// TODO v0.9 perform client-side regular-expression-check if the server does not provide it
				LOGGER.error("Database " + dialect.getSystem() + " does not support regular expression queries");
			}
		}
		if (parent.getDefectType() == null)
			parent.setDefectType(DEFAULT_DEFECT_TYPE);
		parent.setQuery(defectQuery);
	    return defectQuery;
	}

}
