/*
 * (c) Copyright 2010 by Volker Bergmann. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, is permitted under the terms of the
 * GNU General Public License (GPL).
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * WITHOUT A WARRANTY OF ANY KIND. ALL EXPRESS OR IMPLIED CONDITIONS,
 * REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
 * HEREBY EXCLUDED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

package org.databene.dbsanity.report;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import org.databene.commons.StringUtil;
import org.databene.commons.file.FilePrintStream;
import org.databene.dbsanity.model.SanityCheck;
import org.databene.dbsanity.model.SanityCheckFile;
import org.databene.dbsanity.model.SanityCheckFolder;
import org.databene.dbsanity.model.SanityCheckSuite;
import org.databene.dbsanity.model.SanityCheckVerdict;
import org.databene.html.HTMLUtil;

/**
 * Reports the defects per table.<br/><br/>
 * Created: 08.12.2010 20:46:25
 * @since 1.0
 * @author Volker Bergmann
 */
public class DefectsPerTableModule extends AbstractReportModule {
	
	Map<String, Long> tableDefectCounts;

	public DefectsPerTableModule() {
		this.tableDefectCounts = new HashMap<String, Long>();
	}

	@Override
	public int getDashboardColSpan(ReportScope scope) {
		return (scope == ReportScope.ROOT ? 1 : 0);
	}
	
	@Override
	public void renderDashboardView(ReportScope scope, SanityCheckSuite suite,
			FilePrintStream out) {
		div(5, out);
	}
	
	@Override
	public void summary(SanityCheckSuite rootSuite) {
		checkSuite(rootSuite);
		createDetailsPage(rootSuite);
	}

	private void checkSuite(SanityCheckSuite suite) {
		if (suite instanceof SanityCheckFolder) {
			checkFolder((SanityCheckFolder) suite);
		} else
			checkFile((SanityCheckFile) suite);
	}

	private void checkFolder(SanityCheckFolder folder) {
		for (SanityCheckSuite childSuite : folder.getChildSuites().values())
			checkSuite(childSuite);
	}

	private void checkFile(SanityCheckFile file) {
		for (SanityCheck check : file.getChecks().values()) 
			if (check.getVerdict() == SanityCheckVerdict.FAILED) {
				String table = StringUtil.emptyToNull(StringUtil.trim(check.getTable()));
				table = (table != null ? table.toUpperCase() : "[Undefined]");
				Long defectCount = tableDefectCounts.get(table);
				if (defectCount == null)
					defectCount = 0L;
				defectCount += check.getDefectCount();
				tableDefectCounts.put(table, defectCount);
			}
	}
	
	private void createDetailsPage(SanityCheckSuite rootSuite) {
		if (tableDefectCounts.size() == 0)
			return;
		String title = "Defects per Table";
		FilePrintStream out = null;
		File file = new File(rootSuite.getReportFolder(), getDetailsFileName());
		try {
			out = openNewFile(file, title);
			out.println(context.navBar(rootSuite.getReportFolder()));
			div(-1, out);
		} catch (FileNotFoundException e) {
			throw new RuntimeException("Error writing defects per table file: " + file, e);
		} finally {
			super.closeFile(out);
		}
	}

	private void div(int limit, FilePrintStream out) {
		// order defect type counts in descending order
		List<Entry<String, Long>> counts = new ArrayList<Map.Entry<String, Long>>(tableDefectCounts.entrySet());
		Collections.sort(counts, new Comparator<Map.Entry<String, Long>>() {
			public int compare(Entry<String, Long> count1,
					Entry<String, Long> count2) {
				return - count1.getValue().compareTo(count2.getValue());
			}
		});
		startModule(out);
		String title = (limit >= 0 ? "Defects per Table" : context.formatLong(tableDefectCounts.size()) + " Tables with Defects");
		modHeader(title, out);
		startModBody(out);
		if (counts.size() > 0) {
			long maxCount = counts.get(0).getValue();
			// render HTML
			out.println("	<table cellpadding='2px' width='100%'>");
			out.println("		<tr>");
			out.println("			<th align='left' width='20%'>Table</th>");
			out.println("			<th align='left' width='80%'>Count</th>");
			out.println("		</tr>");
			for (int i = 0; (i < limit || limit < 0) && i < counts.size(); i++) {
				Entry<String, Long> count = counts.get(i);
				String percentage = String.valueOf(count.getValue() * 100 / maxCount) + "%";
				out.println("		<tr>");
				out.println("			<td align='left'>" + (count.getKey() != null ? count.getKey() : "unknown") + "</td>");
				out.println("			<td align='left'>");
				ReportUtil.numberBar(count.getValue(), percentage, "#aa0000", context, out);
				out.println("			</td align='left'>");
				out.println("		</tr>");
			}
			if (limit >= 0 && limit < counts.size()) {
				out.println("		<tr>");
				out.println("			<td align='left'>" + HTMLUtil.a(getDetailsFileName(), "[more...]") + "</td>");
				out.println("			<td></td>");
				out.println("		</tr>");
			}
			out.println("	</table>");
		} else {
			out.println("none");
		}
		endModBody(out);
		if (limit > 0)
			modFooter(getDetailsFileName(), out);
		endModule(out);
	}

	public String getDetailsFileName() {
		return "defects_per_table.html";
	}

}
